#pragma once

#include "ReturnCodes_Define.h"
#include "MOTION_DEFINE.h"
#include "COMM_Define.h"
#include <codecvt>
#include <locale>



//------------------------------------------------------------------------------
//					Connection Functions
//------------------------------------------------------------------------------
 int 	FAS_Connect(const wchar_t* sPort, unsigned int dwBaud, int nPortID);

 int 	FAS_OpenPort(int nPortID, unsigned int dwBaud);
 int 	FAS_AttachSlave(int nPortID, unsigned char iSlaveNo);

 void 	FAS_Close(int nPortID);

 int 	FAS_IsSlaveExist(int nPortID, unsigned char iSlaveNo);

//------------------------------------------------------------------------------
//					Log Functions
//------------------------------------------------------------------------------
 void 	FAS_EnableLog(int bEnable);
 void 	FAS_SetLogLevel(enum LOG_LEVEL level);
 int 	FAS_SetLogPath(const wchar_t* lpPath);

 void 	FAS_PrintCustomLog(int nPortID, enum LOG_LEVEL level, const wchar_t* lpszMsg);

//------------------------------------------------------------------------------
//					Info Functions
//------------------------------------------------------------------------------
 int 	FAS_GetSlaveInfo(int nPortID, unsigned char iSlaveNo, unsigned char* pType, char* lpBuff, int nBuffSize);
 int 	FAS_GetMotorInfo(int nPortID, unsigned char iSlaveNo, unsigned char* pType, char* lpBuff, int nBuffSize);
 int 	FAS_GetSlaveInfoEx(int nPortID, unsigned char iSlaveNo, DRIVE_INFO* lpDriveInfo);

//------------------------------------------------------------------------------
//					Parameter Functions
//------------------------------------------------------------------------------
 int 	FAS_SaveAllParameters(int nPortID, unsigned char iSlaveNo);
 int 	FAS_SetParameter(int nPortID, unsigned char iSlaveNo, unsigned char iParamNo, int lParamValue);
 int 	FAS_GetParameter(int nPortID, unsigned char iSlaveNo, unsigned char iParamNo, int* lParamValue);
 int 	FAS_GetROMParameter(int nPortID, unsigned char iSlaveNo, unsigned char iParamNo, int* lRomParam);

//------------------------------------------------------------------------------
//					IO Functions
//------------------------------------------------------------------------------
 int 	FAS_SetIOInput(int nPortID, unsigned char iSlaveNo, unsigned int dwIOSETMask, unsigned int dwIOCLRMask);
 int 	FAS_GetIOInput(int nPortID, unsigned char iSlaveNo, unsigned int* dwIOInput);

 int 	FAS_SetIOOutput(int nPortID, unsigned char iSlaveNo, unsigned int dwIOSETMask, unsigned int dwIOCLRMask);
 int 	FAS_GetIOOutput(int nPortID, unsigned char iSlaveNo, unsigned int* dwIOOutput);

 int 	FAS_GetIOAssignMap(int nPortID, unsigned char iSlaveNo, unsigned char iIOPinNo, unsigned int* dwIOLogicMask, unsigned char* bLevel);
 int 	FAS_SetIOAssignMap(int nPortID, unsigned char iSlaveNo, unsigned char iIOPinNo, unsigned int dwIOLogicMask, unsigned char bLevel);

 int 	FAS_IOAssignMapReadROM(int nPortID, unsigned char iSlaveNo);

//------------------------------------------------------------------------------
//					Servo Driver Control Functions
//------------------------------------------------------------------------------
 int 	FAS_ServoEnable(int nPortID, unsigned char iSlaveNo, int bOnOff);
 int 	FAS_ServoAlarmReset(int nPortID, unsigned char iSlaveNo);
 int 	FAS_StepAlarmReset(int nPortID, unsigned char iSlaveNo, int bReset);
 int 	FAS_BrakeSet(int nPortID, unsigned char iSlaveNo, int bSet, int* nResult);

//------------------------------------------------------------------------------
//					Read Status and Position
//------------------------------------------------------------------------------
 int 	FAS_GetAxisStatus(int nPortID, unsigned char iSlaveNo, unsigned int* dwAxisStatus);
 int 	FAS_GetIOAxisStatus(int nPortID, unsigned char iSlaveNo, unsigned int* dwInStatus, unsigned int* dwOutStatus, unsigned int* dwAxisStatus);
 int 	FAS_GetMotionStatus(int nPortID, unsigned char iSlaveNo, int* lCmdPos, int* lActPos, int* lPosErr, int* lActVel, unsigned short* wPosItemNo);
 int 	FAS_GetAllStatus(int nPortID, unsigned char iSlaveNo, unsigned int* dwInStatus, unsigned int* dwOutStatus, unsigned int* dwAxisStatus, int* lCmdPos, int* lActPos, int* lPosErr, int* lActVel, unsigned short* wPosItemNo);
 int 	FAS_GetAllStatusEx(int nPortID, unsigned char iSlaveNo, unsigned char* pTypes, int* pDatas);

 int 	FAS_SetCommandPos(int nPortID, unsigned char iSlaveNo, int lCmdPos);
 int 	FAS_SetActualPos(int nPortID, unsigned char iSlaveNo, int lActPos);
 int 	FAS_ClearPosition(int nPortID, unsigned char iSlaveNo);
 int 	FAS_GetCommandPos(int nPortID, unsigned char iSlaveNo, int* lCmdPos);
 int 	FAS_GetActualPos(int nPortID, unsigned char iSlaveNo, int* lActPos);
 int 	FAS_GetPosError(int nPortID, unsigned char iSlaveNo, int* lPosErr);
 int 	FAS_GetActualVel(int nPortID, unsigned char iSlaveNo, int* lActVel);

 int 	FAS_GetAlarmType(int nPortID, unsigned char iSlaveNo, unsigned char* nAlarmType);

//------------------------------------------------------------------
//					Motion Functions.
//------------------------------------------------------------------
 int 	FAS_MoveStop(int nPortID, unsigned char iSlaveNo);
 int 	FAS_EmergencyStop(int nPortID, unsigned char iSlaveNo);

 int 	FAS_MovePause(int nPortID, unsigned char iSlaveNo, int bPause);

 int 	FAS_MoveOriginSingleAxis(int nPortID, unsigned char iSlaveNo);
 int 	FAS_MoveSingleAxisAbsPos(int nPortID, unsigned char iSlaveNo, int lAbsPos, unsigned int lVelocity);
 int 	FAS_MoveSingleAxisIncPos(int nPortID, unsigned char iSlaveNo, int lIncPos, unsigned int lVelocity);
 int 	FAS_MoveToLimit(int nPortID, unsigned char iSlaveNo, unsigned int lVelocity, int iLimitDir);
 int 	FAS_MoveVelocity(int nPortID, unsigned char iSlaveNo, unsigned int lVelocity, int iVelDir);

 int 	FAS_PositionAbsOverride(int nPortID, unsigned char iSlaveNo, int lOverridePos);
 int 	FAS_PositionIncOverride(int nPortID, unsigned char iSlaveNo, int lOverridePos);
 int 	FAS_VelocityOverride(int nPortID, unsigned char iSlaveNo, unsigned int lVelocity);

 int 	FAS_MoveLinearAbsPos(int nPortID, unsigned char nNoOfSlaves, unsigned char* iSlavesNo, int* lAbsPos, unsigned int lFeedrate, unsigned short wAccelTime);
 int 	FAS_MoveLinearIncPos(int nPortID, unsigned char nNoOfSlaves, unsigned char* iSlavesNo, int* lIncPos, unsigned int lFeedrate, unsigned short wAccelTime);

 int 	FAS_MoveLinearAbsPos2(int nPortID, unsigned char nNoOfSlaves, unsigned char* iSlavesNo, int* lAbsPos, unsigned int lFeedrate, unsigned short wAccelTime);
 int 	FAS_MoveLinearIncPos2(int nPortID, unsigned char nNoOfSlaves, unsigned char* iSlavesNo, int* lIncPos, unsigned int lFeedrate, unsigned short wAccelTime);

 int 	FAS_MoveCircleAbsPos1(int nPortID, unsigned char nNoOfSlaves, unsigned char* iSlavesNo, int* lplCirEndAbs, int* lplCirCenterAbs, int iDirection, unsigned int lFeedrate, unsigned short wAccelTime, int bSCurve);
 int 	FAS_MoveCircleIncPos1(int nPortID, unsigned char nNoOfSlaves, unsigned char* iSlavesNo, int* lplCirEndInc, int* lplCirCenterInc, int iDirection, unsigned int lFeedrate, unsigned short wAccelTime, int bSCurve);
 int 	FAS_MoveCircleAbsPos2(int nPortID, unsigned char nNoOfSlaves, unsigned char* iSlavesNo, int* lplCirEndAbs, unsigned int lRadius, int iDirection, unsigned int lFeedrate, unsigned short wAccelTime, int bSCurve);
 int 	FAS_MoveCircleIncPos2(int nPortID, unsigned char nNoOfSlaves, unsigned char* iSlavesNo, int* lplCirEndInc, unsigned int lRadius, int iDirection, unsigned int lFeedrate, unsigned short wAccelTime, int bSCurve);
 int 	FAS_MoveCircleAbsPos3(int nPortID, unsigned char nNoOfSlaves, unsigned char* iSlavesNo, int* lplCirCenterAbs, unsigned int nAngle, int iDirection, unsigned int lFeedrate, unsigned short wAccelTime, int bSCurve);
 int 	FAS_MoveCircleIncPos3(int nPortID, unsigned char nNoOfSlaves, unsigned char* iSlavesNo, int* lplCirCenterInc, unsigned int nAngle, int iDirection, unsigned int lFeedrate, unsigned short wAccelTime, int bSCurve);

 int 	FAS_TriggerOutput_RunA(int nPortID, unsigned char iSlaveNo, int bStartTrigger, int lStartPos, unsigned int dwPeriod, unsigned int dwPulseTime);
 int 	FAS_TriggerOutput_Status(int nPortID, unsigned char iSlaveNo, unsigned char* bTriggerStatus);

 int 	FAS_SetTriggerOutputEx(int nPortID, unsigned char iSlaveNo, unsigned char nOutputNo, unsigned char bRun, unsigned short wOnTime, unsigned char nTriggerCount, int* arrTriggerPosition);
 int 	FAS_GetTriggerOutputEx(int nPortID, unsigned char iSlaveNo, unsigned char nOutputNo, unsigned char* bRun, unsigned short* wOnTime, unsigned char* nTriggerCount, int* arrTriggerPosition);

 int 	FAS_MovePush(int nPortID, unsigned char iSlaveNo, unsigned int dwStartSpd, unsigned int dwMoveSpd, int lPosition, unsigned short wAccel, unsigned short wDecel, unsigned short wPushRate, unsigned int dwPushSpd, int lEndPosition, unsigned short wPushMode);
 int 	FAS_GetPushStatus(int nPortID, unsigned char iSlaveNo, unsigned char* nPushStatus);

//------------------------------------------------------------------
//					Ex-Motion Functions.
//------------------------------------------------------------------
 int 	FAS_MoveSingleAxisAbsPosEx(int nPortID, unsigned char iSlaveNo, int lAbsPos, unsigned int lVelocity, MOTION_OPTION_EX* lpExOption);
 int 	FAS_MoveSingleAxisIncPosEx(int nPortID, unsigned char iSlaveNo, int lIncPos, unsigned int lVelocity, MOTION_OPTION_EX* lpExOption);
 int 	FAS_MoveVelocityEx(int nPortID, unsigned char iSlaveNo, unsigned int lVelocity, int iVelDir, VELOCITY_OPTION_EX* lpExOption);

//------------------------------------------------------------------
//					All-Motion Functions.
//------------------------------------------------------------------
 int 	FAS_AllMoveStop(int nPortID);
 int 	FAS_AllEmergencyStop(int nPortID);
 int 	FAS_AllMoveOriginSingleAxis(int nPortID);
 int 	FAS_AllMoveSingleAxisAbsPos(int nPortID, int lAbsPos, unsigned int lVelocity);
 int 	FAS_AllMoveSingleAxisIncPos(int nPortID, int lIncPos, unsigned int lVelocity);

//------------------------------------------------------------------
//					Position Table Functions.
//------------------------------------------------------------------
 int 	FAS_PosTableReadItem(int nPortID, unsigned char iSlaveNo, unsigned short wItemNo, LPITEM_NODE lpItem);
 int 	FAS_PosTableWriteItem(int nPortID, unsigned char iSlaveNo, unsigned short wItemNo, LPITEM_NODE lpItem);
 int 	FAS_PosTableWriteROM(int nPortID, unsigned char iSlaveNo);
 int 	FAS_PosTableReadROM(int nPortID, unsigned char iSlaveNo);
 int 	FAS_PosTableRunItem(int nPortID, unsigned char iSlaveNo, unsigned short wItemNo);

 int 	FAS_PosTableReadOneItem(int nPortID, unsigned char iSlaveNo, unsigned short wItemNo, unsigned short wOffset, int* lPosItemVal);
 int 	FAS_PosTableWriteOneItem(int nPortID, unsigned char iSlaveNo, unsigned short wItemNo, unsigned short wOffset, int lPosItemVal);

 int 	FAS_PosTableSingleRunItem(int nPortID, unsigned char iSlaveNo, int bNextMove, unsigned short wItemNo);

//------------------------------------------------------------------
//					Gap Control Functions.
//------------------------------------------------------------------
 int 	FAS_GapControlEnable(int nPortID, unsigned char iSlaveNo, unsigned short wItemNo, int lGapCompSpeed, int lGapAccTime, int lGapDecTime, int lGapStartSpeed);
 int 	FAS_GapControlDisable(int nPortID, unsigned char iSlaveNo);
 int 	FAS_IsGapControlEnable(int nPortID, unsigned char iSlaveNo, int* bIsEnable, unsigned short* wCurrentItemNo);

 int 	FAS_GapControlGetADCValue(int nPortID, unsigned char iSlaveNo, int* lADCValue);
 int 	FAS_GapOneResultMonitor(int nPortID, unsigned char iSlaveNo, unsigned char* bUpdated, int* iIndex, int* lGapValue, int* lCmdPos, int* lActPos, int* lCompValue, int* lReserved);

//------------------------------------------------------------------
//					Alarm Type History Functions.
//------------------------------------------------------------------
 int 	FAS_GetAlarmLogs(int nPortID, unsigned char iSlaveNo, ALARM_LOG* pAlarmLog);
 int 	FAS_ResetAlarmLogs(int nPortID, unsigned char iSlaveNo);

//------------------------------------------------------------------
//					I/O Module Functions.
//------------------------------------------------------------------
 int 	FAS_GetInput(int nPortID, unsigned char iSlaveNo, unsigned int* uInput, unsigned int* uLatch);

 int 	FAS_ClearLatch(int nPortID, unsigned char iSlaveNo, unsigned int uLatchMask);
 int 	FAS_GetLatchCount(int nPortID, unsigned char iSlaveNo, unsigned char iInputNo, unsigned int* uCount);
 int 	FAS_GetLatchCountAll(int nPortID, unsigned char iSlaveNo, unsigned int** ppuAllCount);
 int 	FAS_GetLatchCountAll32(int nPortID, unsigned char iSlaveNo, unsigned int** ppuAllCount);
 int 	FAS_ClearLatchCount(int nPortID, unsigned char iSlaveNo, unsigned int uInputMask);

 int 	FAS_GetOutput(int nPortID, unsigned char iSlaveNo, unsigned int* uOutput, unsigned int* uStatus);
 int 	FAS_SetOutput(int nPortID, unsigned char iSlaveNo, unsigned int uSetMask, unsigned int uClearMask);

 int 	FAS_SetTrigger(int nPortID, unsigned char iSlaveNo, unsigned char uOutputNo, TRIGGER_INFO* pTrigger);
 int 	FAS_SetRunStop(int nPortID, unsigned char iSlaveNo, unsigned int uRunMask, unsigned int uStopMask);
 int 	FAS_GetTriggerCount(int nPortID, unsigned char iSlaveNo, unsigned char uOutputNo, unsigned int* uCount);

 int 	FAS_GetIOLevel(int nPortID, unsigned char iSlaveNo, unsigned int* uIOLevel);
 int 	FAS_SetIOLevel(int nPortID, unsigned char iSlaveNo, unsigned int uIOLevel);
 int 	FAS_LoadIOLevel(int nPortID, unsigned char iSlaveNo);
 int 	FAS_SaveIOLevel(int nPortID, unsigned char iSlaveNo);

 int 	FAS_GetInputFilter(int nPortID, unsigned char iSlaveNo, unsigned short* filter);
 int 	FAS_SetInputFilter(int nPortID, unsigned char iSlaveNo, unsigned short filter);

 int 	FAS_GetIODirection(int nPortID, unsigned char iSlaveNo, unsigned int* direction);
 int 	FAS_SetIODirection(int nPortID, unsigned char iSlaveNo, unsigned int direction);

//------------------------------------------------------------------
//					Ezi-IO AD Functions
//------------------------------------------------------------------
 int 	FAS_SetADConfig(int nPortID, unsigned char iSlaveNo, unsigned char channel, enum AD_DATA_TYPE type, int value, int* recv);
 int 	FAS_GetADConfig(int nPortID, unsigned char iSlaveNo, unsigned char channel, enum AD_DATA_TYPE type, int* value);

 int 	FAS_LoadADConfig(int nPortID, unsigned char iSlaveNo);
 int 	FAS_SaveADConfig(int nPortID, unsigned char iSlaveNo);

 int 	FAS_ReadADValue(int nPortID, unsigned char iSlaveNo, unsigned char channel, short* advalue);
 int 	FAS_ReadADAllValue(int nPortID, unsigned char iSlaveNo, unsigned char offset, AD_BUFFER* adbuffer);

 int 	FAS_GetAllADResult(int nPortID, unsigned char iSlaveNo, AD_RESULT* result);
 int 	FAS_GetADResult(int nPortID, unsigned char iSlaveNo, unsigned char channel, float* adresult);
 int 	FAS_SetADRange(int nPortID, unsigned char iSlaveNo, unsigned char channel, enum AD_RANGE range);
